Unit Unit1;

{$MODE objfpc}{$H+}

Interface

Uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, TAGraph,
  TATransformations, TAIntervalSources, Types, TASeries, TAChartUtils;

Type

  TDataRec = Record
    x, y: Double;
  End;


  { TForm1 }

  TForm1 = Class(TForm)
    BatteryPercentSeries: TLineSeries;
    Button1: TButton;
    Button2: TButton;
    Chart1: TChart;
    Chart2: TChart;
    Chart2LineSeries1: TLineSeries;
    Chart2LineSeries2: TLineSeries;
    ChartAxisTransformations1: TChartAxisTransformations;
    ChartAxisTransformations1AutoScaleAxisTransform1: TAutoScaleAxisTransform;
    ChartAxisTransformations1AutoScaleAxisTransform2: TAutoScaleAxisTransform;
    ChartAxisTransformations2: TChartAxisTransformations;
    ChartAxisTransformations2AutoScaleAxisTransform1: TAutoScaleAxisTransform;
    ChartAxisTransformations2AutoScaleAxisTransform2: TAutoScaleAxisTransform;
    ChartAxisTransformations3: TChartAxisTransformations;
    ChartAxisTransformations4: TChartAxisTransformations;
    DateTimeIntervalChartSource1: TDateTimeIntervalChartSource;
    ENBWPowerSeries: TUserDrawnSeries;
    Procedure Button1Click(Sender: TObject);
    Procedure Button2Click(Sender: TObject);
    Procedure ENBWPowerSeriesDraw(ACanvas: TCanvas; Const ARect: TRect);
    Procedure ENBWPowerSeriesGetBounds(Var ABounds: TDoubleRect);
    Procedure FormClose(Sender: TObject; Var CloseAction: TCloseAction);
    Procedure FormCreate(Sender: TObject);
  private
    Extend: TDoubleRect; // Das Bound Rect von data
    Data: Array Of TDataRec;
    Pen: TPen;
    Procedure ClearLines();
    Procedure AddLineValue(x, y: Double);
  public

  End;

Var
  Form1: TForm1;

Implementation

{$R *.lfm}

Uses
  math, TAGeometry;

Const
  COLOR_BELOW = clLime;
  COLOR_ABOVE = clred;

Function Intersect(p1, p2: TDoublePoint; y: Double): Double;
Begin
  If p1.y = p2.y Then Begin
    Result := (p2.x + p1.x) / 2;
  End
  Else Begin
    Result := ((p2.x - p1.x) * (y - p1.y)) / (p2.y - p1.y) + p1.x;
  End;
End;

Function ToPoint(Value: TDoublePoint): TPoint;
Begin
  result.x := round(value.x);
  result.Y := round(value.Y);
End;

{ TForm1 }

Procedure TForm1.ENBWPowerSeriesDraw(ACanvas: TCanvas; Const ARect: TRect);
  Function GetColor(y, yref: Double): TColor;
  Begin
    If y > yref Then
      Result := COLOR_BELOW
    Else
      Result := COLOR_ABOVE;
  End;

Var
  i: Integer;
  gp: TDoublePoint;
  p1, p2, p3: TDoublePoint;
  yref: Integer;
Begin
  If Not assigned(data) Then exit;
  Chart1.Drawer.Pen := Pen;
  yref := Chart1.YGraphToImage(0);
  gp := DoublePoint(Data[0].X, Data[0].Y);
  p1 := DoublePoint(Chart1.GraphToImage(gp));
  For i := 1 To High(Data) Do Begin
    gp := DoublePoint(Data[i].X, Data[i].Y);
    p2 := DoublePoint(Chart1.GraphToImage(gp));
    If (p2.y - yref) * (p1.y - yref) <= 0 Then Begin
      p3 := DoublePoint(Intersect(p1, p2, yref), yref);
      Chart1.Drawer.SetPenParams(psSolid, GetColor(p1.y, yref), 2);
      Chart1.Drawer.Line(ToPoint(p1), ToPoint(p3));
      Chart1.Drawer.SetPenParams(psSolid, GetColor(p2.y, yref), 2);
      Chart1.Drawer.Line(ToPoint(p3), ToPoint(p2));
    End
    Else Begin
      Chart1.Drawer.SetPenParams(psSolid, GetColor(p2.y, yref), 2);
      Chart1.Drawer.Line(ToPoint(p1), ToPoint(p2));
    End;
    p1 := p2;
  End;
End;

Procedure TForm1.Button1Click(Sender: TObject);
Var
  i: Integer;
Begin
  ClearLines;
  BatteryPercentSeries.Clear;
  For i := 0 To 100 Do Begin
    AddLineValue(i / 100, random(5000) - 4000);
    BatteryPercentSeries.AddXY(i / 100, random(100));
  End;
  Chart1.Invalidate;
End;

Procedure TForm1.Button2Click(Sender: TObject);
Var
  i: Integer;
Begin
  Chart2LineSeries1.Clear;
  Chart2LineSeries2.Clear;
  For i := 0 To 100 Do Begin
    Chart2LineSeries1.addxy(i / 100, random(5000) - 4000);
    Chart2LineSeries2.AddXY(i / 100, random(100));
  End;
  Chart1.Invalidate;
End;

Procedure TForm1.ENBWPowerSeriesGetBounds(Var ABounds: TDoubleRect);
Begin
  ABounds := Extend;
End;

Procedure TForm1.FormClose(Sender: TObject; Var CloseAction: TCloseAction);
Begin
  pen.free;
  pen := Nil;
End;

Procedure TForm1.FormCreate(Sender: TObject);
Begin
  Pen := TPen.Create;
End;

Procedure TForm1.ClearLines;
Begin
  BatteryPercentSeries.Clear;
  setlength(Data, 0);
  Extend.a.x := 0;
  Extend.a.y := 0;
  Extend.b.x := 0;
  Extend.b.y := 0;
End;

Procedure TForm1.AddLineValue(x, y: Double);
Begin
  setlength(data, high(data) + 2);
  data[high(data)].x := x;
  data[high(data)].y := y;
  If length(data) = 1 Then Begin
    Extend.a.x := data[0].x;
    Extend.a.Y := data[0].y;
    Extend.b.x := data[0].x;
    Extend.b.y := data[0].y;
  End
  Else Begin
    Extend.a.x := min(Extend.a.x, x);
    Extend.a.Y := min(Extend.a.Y, y);
    Extend.b.x := max(Extend.b.x, x);
    Extend.b.Y := max(Extend.b.Y, y);
  End;
End;

End.

